#pragma once

#include "NPCommonUI.h"
#include <atlwin.h>
#include <atlcoll.h>

#include "Common.h"
#include "RootContainer.h"
#include "ActionEvent.h"
#include "WindowEvent.h"
#include "Object.h"

#define UM_PRETRANSLATE WM_USER + 101
#define UM_REFRESH WM_USER + 1024

class NPCOMMONUI_API CRootWindow : public Object
{
public:
	CRootWindow(void);
	virtual ~CRootWindow(void);

	void	SetAlpha(int alpha){ m_alpha = alpha; }
	HWND	GetWindow() { return m_hWndBase; }
	/**
	* @brief	the function retrieves the root container
	* @return	the pointer of root container
	*/
	CRootContainer*	GetRootContainer() { return &m_rootContainer; }

	/**
	* @brief 	pretranslate message
	* @param	pMsg		massage struct
	* @return	If the msaage is processed , return the TRUE or FALSE
	*/
	//BOOL PreTranslateMessage(MSG* pMsg){ return m_rootContainer.PreTranslateMessage(pMsg); }

	void SetCloseAction(IWindowListener* pAction)
	{
		m_closeAction = pAction;
	}
	void SetMinimizeAction(IActionListener* pAction)
	{
		m_minimizeAction = pAction;
	}
	void SetMaximizeAction(IActionListener* pAction)
	{
		m_maximizeAction = pAction;
	}
	void SetRoundRegion(int round);
	void SetTaskbar(BOOL bTaskbar);
	BOOL IsTaskbar() { return m_bTaskbar; }
	void SetCenterWindow(BOOL bCenterWindow);
	BOOL IsCenterWindow() { return m_bCenterWindow; }

	void SetDataContext(IDataContext* datacontext);
	IDataContext* GetDataContext() { return m_dataContext; }

protected:
	virtual void OnCreate() {}
	virtual void OnDestroy() {}
	virtual void Draw(HDC hDC);
	virtual int GetDoubleBufferBitCount() { return 32; }
	
	BOOL _UpdateBufferImage(HDC hDC, int cx, int cy);
	void SetWindow(HWND hWnd) { m_hWndBase = hWnd; }

	void _DrawLayer(HDC hDC);

	BEGIN_MSG_MAP(CRootWindow)
		MESSAGE_HANDLER(UM_PRETRANSLATE, CRootWindow::OnPreTranslateMessage)
		MESSAGE_HANDLER(WM_MOUSEMOVE, CRootWindow::OnMouseMove)
		
		CHAIN_MSG_MAP_MEMBER(m_rootContainer)

		MESSAGE_HANDLER(WM_MOUSELEAVE, CRootWindow::OnMouseLeave)
		MESSAGE_HANDLER(WM_PAINT, CRootWindow::OnPaint)
		MESSAGE_HANDLER(WM_ERASEBKGND, CRootWindow::OnEraseBkgnd)
		MESSAGE_HANDLER(WM_SIZE, CRootWindow::OnSize)
		MESSAGE_HANDLER(WM_KILLFOCUS, CRootWindow::OnKillFocus)
		MESSAGE_HANDLER(WM_SYSCOMMAND, CRootWindow::OnSyscommand)
		MESSAGE_HANDLER(WM_DESTROY, CRootWindow::OnDestroy)
		
		MESSAGE_HANDLER(UM_REFRESH, CRootWindow::OnUIRefresh)
	END_MSG_MAP()

	///< double buffered image
	HBITMAP			m_hBitmapDB;
	///< double buffered image info
	BITMAP			m_bmDB;
	///< buffer dib buffer
	LPBYTE	m_pbmpBits;

	CRootContainer	m_rootContainer;

	IWindowListener* m_closeAction;
	IActionListener* m_minimizeAction;
	IActionListener* m_maximizeAction;
	IDataContext* m_dataContext;
	int		m_RoundRegion;
	HWND	m_hWndBase;

private:
	int		m_alpha;
	BOOL	m_bTracking;
	BOOL	m_bTaskbar;
	BOOL	m_bCenterWindow;
	byte b;

protected:
	DECLARE_MESSAGE_HANDLER(OnPreTranslateMessage);
	DECLARE_MESSAGE_HANDLER(OnMouseMove);
	DECLARE_MESSAGE_HANDLER(OnPaint);
	DECLARE_MESSAGE_HANDLER(OnEraseBkgnd);
	DECLARE_MESSAGE_HANDLER(OnSize);
	DECLARE_MESSAGE_HANDLER(OnMouseLeave);
	DECLARE_MESSAGE_HANDLER(OnKillFocus);
	DECLARE_MESSAGE_HANDLER(OnSyscommand);
	DECLARE_MESSAGE_HANDLER(OnDestroy);
	DECLARE_MESSAGE_HANDLER(OnUIRefresh);
};
